import 'package:flutter/material.dart';

extension WhenAsyncSnapshotExtension<T> on AsyncSnapshot<T> {
  R when<R>({
    required R Function(T data) data,
    required R Function(Object, StackTrace?, T? previous) error,
    required R Function(T? previous) loading,
  }) {
    if (hasError) {
      return error(this.error!, stackTrace, this.data);
    }

    if (connectionState != ConnectionState.done || !hasData) {
      return loading(this.data);
    }

    return data(this.data!);
  }

  R maybeWhen<R>({
    R Function(T data)? data,
    R Function(Object, StackTrace?, T? previous)? error,
    R Function(T? previous)? loading,
    required R Function() orElse,
  }) {
    if (hasError && error != null) {
      return error(this.error!, stackTrace, this.data);
    }

    if ((connectionState != ConnectionState.done || !hasData) &&
        loading != null) {
      return loading(this.data);
    }

    if (hasData && data != null) {
      return data(this.data!);
    }

    return orElse();
  }
}
