import 'package:firebase_hooks_core/firebase_hooks_core.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  group('When on async snapshot', () {
    test('Nothing should equal loading', () {
      final stuff = const AsyncSnapshot<void>.nothing().when(
        data: (data) => 'foo',
        error: (error, stack, previous) => 'bar',
        loading: (previous) => 'foobar',
      );

      expect(stuff, 'foobar');
    });

    test('With data should be called', () {
      final stuff =
          const AsyncSnapshot.withData(ConnectionState.done, 'fighters').when(
        data: (data) => 'foo' + data,
        error: (error, stack, previous) => 'bar',
        loading: (previous) => 'foobar',
      );

      expect(stuff, 'foofighters');
    });

    test('With data and loading should be called', () {
      final stuff =
          const AsyncSnapshot.withData(ConnectionState.active, 'fighters').when(
        data: (data) {
          return 'loading' + data;
        },
        error: (error, stack, previous) => 'bar',
        loading: (previous) => 'foobar',
      );

      expect(stuff, 'loadingfighters');
    });

    test('Error should be called', () {
      final stuff =
          const AsyncSnapshot<void>.withError(ConnectionState.active, 'shucks')
              .when(
        data: (data) => 'foo',
        error: (error, stack, previous) => 'bar',
        loading: (previous) => 'foobar',
      );

      expect(stuff, 'bar');
    });
  });

  group('MaybeWhen on asyncSnapshot', () {
    test('Nothing should equal loading', () {
      final stuff = const AsyncSnapshot<void>.nothing().maybeWhen(
        data: (data) => 'foo',
        error: (error, stack, previous) => 'bar',
        loading: (previous) => 'foobar',
        orElse: () => 'notme',
      );

      expect(stuff, 'foobar');
    });

    test('With data should be called', () {
      throw UnimplementedError();
    });

    test('With data and loading should be called', () {
      throw UnimplementedError();
    });
  });
}
